@@ -72,17 +72,17 @@ bugzilla_product_insert_component (BugzillaProduct *prod, BugzillaComponent *com
 /* they aren't anymore, and my iconv on solaris isn't doing UTF-8 for
  * some reason */
 static char *
-gify (char *x)
+gify (guchar *x)
 {
 	char *g;
-	g = g_convert (x, strlen (x), "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+	g = g_convert ((char *)x, strlen ((char *)x), "UTF-8", "ISO8859-1", NULL, NULL, NULL);
 	if (!g)
-		g = g_strdup (x);
+		g = g_strdup ((char *)x);
 	xmlFree (x);
 	return g;
 }
 	
-#define XML_NODE_GET_PROP(node, name) (gify (xmlGetProp ((node), (name))))
+#define XML_NODE_GET_PROP(node, name) (gify (xmlGetProp ((node), (guchar *)(name))))
 #define XML_NODE_GET_CONTENT(node)    (gify (xmlNodeGetContent ((node))))
 
 static void
@@ -98,17 +98,17 @@ load_mostfreq_xml (BugzillaBTS *bts, xmlDoc *doc)
 	for (prod = xmlDocGetRootElement (doc)->children; prod; prod = prod->next) {
 		d(g_print ("\t%s\n", prod->name));
 
-		if (strcmp (prod->name, "product"))
+		if (strcmp ((char *)prod->name, "product"))
 			continue;
 
 		for (comp = prod->children; comp; comp = comp->next) {
 			d(g_print ("\t\t%s\n", comp->name));
-			if (strcmp (comp->name, "component"))
+			if (strcmp ((char *)comp->name, "component"))
 				continue;
 
 			for (bug = comp->children; bug; bug = bug->next) {
 				d(g_print ("\t\t\t%s\n", bug->name));
-				if (strcmp (bug->name, "bug"))
+				if (strcmp ((char *)bug->name, "bug"))
 					continue;
 				
 				bbug = g_new0 (BugzillaBug, 1);
@@ -120,9 +120,9 @@ load_mostfreq_xml (BugzillaBTS *bts, xmlDoc *doc)
 				for (node = bug->children; node; node = node->next) {
 					d(g_print ("\t\t\t\t%s\n", node->name));
 
-					if (!strcmp (node->name, "desc"))
+					if (!strcmp ((char *)node->name, "desc"))
 						bbug->desc = XML_NODE_GET_CONTENT (node);
-					else if (!strcmp (node->name, "url"))
+					else if (!strcmp ((char *)node->name, "url"))
 						bbug->url = XML_NODE_GET_CONTENT (node);
 				}
 				bts->bugs = g_slist_prepend (bts->bugs, bbug);
@@ -146,19 +146,19 @@ load_config_xml (BugzillaBTS *bts, xmlDoc *doc)
 
 	for (node = xmlDocGetRootElement (doc)->children; node; node = node->next) {
 		d(g_print ("\t%s\n", node->name));
-		if (!strcmp (node->name, bts->severity_node)) {
+		if (!strcmp ((char *)node->name, bts->severity_node)) {
 			for (cur = node->children; cur; cur = cur->next) {
 				d(g_print ("\t\t%s\n", cur->name));
-				if (strcmp (cur->name, bts->severity_item))
+				if (strcmp ((char *)cur->name, bts->severity_item))
 					continue;
 				bts->severities = g_slist_append (
 					bts->severities,
 					XML_NODE_GET_CONTENT (cur));
 			}
-		} else if (!strcmp (node->name, "opsys_list")) {
+		} else if (!strcmp ((char *)node->name, "opsys_list")) {
 			for (cur = node->children; cur; cur = cur->next) {
 				d(g_print ("\t\t%s\n", cur->name));
-				if (strcmp (cur->name, "opsys"))
+				if (strcmp ((char *)cur->name, "opsys"))
 					continue;
 				bts->opsys = g_slist_append (
 					bts->opsys,
@@ -185,10 +185,10 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 
 	for (node = xmlDocGetRootElement (doc)->children; node; node = node->next) {
 		d(g_print ("\t%s\n", node->name));
-		if (!strcmp (node->name, "product")) {
+		if (!strcmp ((char *)node->name, "product")) {
 
-			tmp = xmlGetProp (node, "disallownew");
-			if (tmp && !strcmp(tmp, "1")) {
+			tmp = xmlGetProp (node, (guchar *)"disallownew");
+			if (tmp && !strcmp((char *)tmp, "1")) {
 				xmlFree (tmp);
 				continue;
 			}
@@ -208,11 +208,11 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 
 			for (cur = node->children; cur; cur = cur->next) {
 				d(g_print ("\t\t%s\n", node->name));
-				if (!strcmp (cur->name, "component")) {
+				if (!strcmp ((char *)cur->name, "component")) {
 					/* This is a workaround for bad generated products.xml
 					   We really need to strict validation on bugzilla side
 					   or rewrote the script generating products.xml*/
-					tmp = xmlGetProp (cur, "value");
+					tmp = xmlGetProp (cur, (guchar *)"value");
 					if (!tmp) {
 						continue;
 					} 
@@ -224,7 +224,7 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 					comp->description = XML_NODE_GET_PROP (cur, "description");
 
 					bugzilla_product_insert_component (prod, comp);
-				} else if (!strcmp (cur->name, "version")) {
+				} else if (!strcmp ((char *)cur->name, "version")) {
 					prod->versions = g_slist_append(prod->versions, XML_NODE_GET_PROP (cur, "value"));
 				}
 			}
@@ -700,12 +700,75 @@ mostfreq_motion (GtkWidget *w, GdkEventMotion *event, gpointer data)
 	return FALSE;
 }
 
+gint
+sort_mostfreq_list (GtkTreeModel *model,
+		    GtkTreeIter *a,
+		    GtkTreeIter *b,
+		    gpointer user_data)
+{
+	gchar *value1 = NULL;
+	gchar *value2 = NULL;
+	glong number1 = -1;
+	glong number2 = -1;
+	gint ret = 0;
+
+	gtk_tree_model_get (model, a, GPOINTER_TO_INT (user_data), &value1, -1);
+	gtk_tree_model_get (model, b, GPOINTER_TO_INT (user_data), &value2, -1);
+
+	switch (GPOINTER_TO_INT (user_data))
+	  {
+	  case MOSTFREQ_PRODUCT:
+	  case MOSTFREQ_COMPONENT:
+	  case MOSTFREQ_DESC:
+	    if (value1 == NULL || value2 == NULL)
+	      {
+		if (value1 == NULL && value2 == NULL)
+		  break;
+		ret = (value1 == NULL) ? -1 : 1;
+	      }
+	    else
+	      {
+		ret = g_utf8_collate (value1, value2);
+	      }
+	    
+	    break;
+
+	  case MOSTFREQ_ID:
+	    if (value1 == NULL || value2 == NULL)
+	      {
+		if (value1 == NULL && value2 == NULL)
+		  break;
+		if (value1 == NULL)
+		  {
+		    number1 = 0;
+		  }
+		if (value2 == NULL)
+		  {
+		    number2 = 0;
+		  }
+	      }
+	    else
+	      {
+		number1 = atol (value1);
+		number2 = atol (value2);
+		if (number1 == number2)
+		  break;
+	      }
+	    ret = (number1 > number2) ? 1 : -1;
+	    break;
+	  }
+	g_free (value1);
+	g_free (value2);
+	return ret;  
+}
+
 static void
 create_mostfreq_list (void)
 {
 	GtkListStore *model;
 	GtkTreeView *view;
 	GtkCellRenderer *ren;
+	GtkTreeViewColumn *column;
 
 	view = GTK_TREE_VIEW (GET_WIDGET ("mostfreq-list"));
 
@@ -737,17 +800,54 @@ create_mostfreq_list (void)
 						     "text", MOSTFREQ_PRODUCT,
 						     NULL);
 
-	/*ren = gtk_cell_renderer_text_new ();*/
 	gtk_tree_view_insert_column_with_attributes (view, -1,
 						     _("Component"), ren,
 						     "text", MOSTFREQ_COMPONENT,
 						     NULL);
 
-	/*ren = gtk_cell_renderer_text_new ();*/
 	gtk_tree_view_insert_column_with_attributes (view, -1,
 						     _("Description"), ren,
 						     "text", MOSTFREQ_DESC,
 						     NULL);
+
+ 	column = gtk_tree_view_get_column (view, 0);
+ 	gtk_tree_view_column_set_sort_column_id (column, MOSTFREQ_ID);
+   	column = gtk_tree_view_get_column (view, 1);
+  	gtk_tree_view_column_set_sort_column_id (column, MOSTFREQ_PRODUCT);
+  	column = gtk_tree_view_get_column (view, 2);
+ 	gtk_tree_view_column_set_sort_column_id (column, MOSTFREQ_COMPONENT);
+ 	column = gtk_tree_view_get_column (view, 3);
+ 	gtk_tree_view_column_set_sort_column_id (column, MOSTFREQ_DESC);
+
+	
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
+					      MOSTFREQ_ID,
+					      GTK_SORT_ASCENDING);
+
+
+	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
+					 MOSTFREQ_ID,
+					 sort_mostfreq_list,
+					 GINT_TO_POINTER (MOSTFREQ_ID),
+					 NULL);
+
+ 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
+					 MOSTFREQ_PRODUCT,
+					 sort_mostfreq_list,
+					 GINT_TO_POINTER (MOSTFREQ_PRODUCT),
+					 NULL);
+
+	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
+					 MOSTFREQ_COMPONENT,
+					 sort_mostfreq_list,
+					 GINT_TO_POINTER (MOSTFREQ_COMPONENT),
+					 NULL);
+
+	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
+					 MOSTFREQ_DESC,
+					 sort_mostfreq_list,
+					 GINT_TO_POINTER (MOSTFREQ_DESC),
+					 NULL);
 }
 
 gboolean
@@ -1300,31 +1400,25 @@ add_component (gpointer key, BugzillaComponent *comp, GtkListStore *store)
 static void
 update_severity (GtkWidget *w, gpointer data)
 {
-	druid_data.severity = (char *)data;
+  	g_free (druid_data.severity);
+	druid_data.severity = gtk_combo_box_get_active_text (GTK_COMBO_BOX (w));    
 }
 
 static void
 update_version (GtkWidget *w, gpointer data)
 {
-	druid_data.version = (char *)data;
+	g_free (druid_data.version);
+	druid_data.version = gtk_combo_box_get_active_text (GTK_COMBO_BOX (w));
 }
 
 
 static void
-add_severity (char *s, GtkMenu *m)
+add_severity (char *s , GtkComboBox * c)
 {
-	GtkWidget *w;
-
-	w = gtk_menu_item_new_with_label (s);
-	g_signal_connect (G_OBJECT (w), "activate",
-			  G_CALLBACK (update_severity),
-			  s);
-	gtk_widget_show (w);
-	gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (c), s);
 	if (!strcasecmp (s, "normal") || !strcasecmp (s, "unknown")) {
-		gtk_menu_item_activate (GTK_MENU_ITEM (w));
-		gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("severity-list")), 
-					     g_slist_index (druid_data.product->bts->severities, s));
+		gtk_combo_box_set_active (GTK_COMBO_BOX (c),
+					  g_slist_index (druid_data.product->bts->severities, s));
 	}
 }
 
@@ -1366,24 +1460,21 @@ void
 update_version_menu (BugzillaProduct *prod)
 {
 	GSList *versions;
-	GtkWidget *w;
-	GtkWidget *m, *c;
+	GtkWidget *c;
+	GtkListStore *store;
 
 	versions = prod->versions;
 
-	m = gtk_menu_new ();
 	c = GET_WIDGET ("the-version-menu");
-        gtk_option_menu_set_menu (GTK_OPTION_MENU (c), m);
-
+	store = (GtkListStore * ) gtk_combo_box_get_model (GTK_COMBO_BOX (c));
+	gtk_list_store_clear (store);
 	while (versions != NULL) {
-		w = gtk_menu_item_new_with_label ((gchar*)versions->data);
-		g_signal_connect (G_OBJECT (w), "activate",
-				  G_CALLBACK (update_version),
-				  versions->data);
-		gtk_widget_show (w);
-		gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (c), (gchar*)versions->data);
 		versions = versions->next;
 	}
+	g_signal_connect (G_OBJECT (c), "changed",
+			  G_CALLBACK (update_version),
+			  NULL);
 }
 
 
@@ -1423,7 +1514,7 @@ void
 bugzilla_product_add_components_to_clist (BugzillaProduct *prod)
 {
 	GtkWidget *m, *c;
-	GtkTreeView *w;
+ 	GtkTreeView *w;
 	GtkListStore *store;
 
 	w = GTK_TREE_VIEW (GET_WIDGET ("component-list"));
@@ -1437,10 +1528,12 @@ bugzilla_product_add_components_to_clist (BugzillaProduct *prod)
 
 	gtk_tree_view_columns_autosize (w);
 
-	m = gtk_menu_new ();
 	c = GET_WIDGET ("severity-list");
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (c), m);
-	g_slist_foreach (prod->bts->severities, (GFunc)add_severity, m);
+	store = (GtkListStore *) gtk_combo_box_get_model (GTK_COMBO_BOX (c));
+	gtk_list_store_clear (store);
+	g_slist_foreach (prod->bts->severities, (GFunc)add_severity, c);
+	g_signal_connect (G_OBJECT (c), "changed",
+			  G_CALLBACK (update_severity), NULL);
 }
 
 static void
@@ -1461,7 +1554,6 @@ add_mostfreq (BugzillaBug *bug, GtkListStore *store)
 gboolean
 bugzilla_add_mostfreq (BugzillaBTS *bts)
 {
-	/* GtkWidget *m, *c; */
 	GtkTreeView *w;
 	GtkListStore *store;
 	GtkTreeSelection *selection;
