@@ -25,31 +25,30 @@
 #include "libglade-buddy.h"
 #include "save-buddy.h"
 
-#include <gnome.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+#include <gtk/gtkaboutdialog.h>
+#include <gtk/gtkcombobox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtktextview.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtktreeview.h>
+
+#include <libgnome/gnome-help.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+#include <unistd.h>
+#include <fcntl.h>
+#include <signal.h>
 #include <sys/types.h>
 #include <sysexits.h>
 #include <sys/wait.h>
 
-#if 0
-static char *help_pages[] = {
-	"index.html",
-	"debug-info.html",
-	"description.html",
-	"updating.html",
-	"product.html",
-	"component.html",
-	"system-config.html",
-	"submit-report.html",
-	"summary.html",
-	NULL
-};
-#endif
-
 static char *state_title[] = {
 	N_("Welcome to Bug Buddy"),
 	N_("Collecting debug info"),
@@ -97,18 +96,7 @@ mail_write_template (void)
 		TEXT_BUG_I18N,	TEXT_BUG_DOC,	TEXT_BUG_REQUEST
 	};
 
-	GtkTextBuffer *desc_buffer;
-	GtkTextIter iter;
-
 	buddy_set_text ("desc-text", templates[druid_data.bug_type]);
-
-	/* set the cursor at the beginning of the second line
-	* in the description page */
-	desc_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW
-						(GET_WIDGET ("desc-text")));
-	gtk_text_buffer_get_iter_at_line (desc_buffer, &iter, 1);
-	gtk_text_buffer_place_cursor (desc_buffer, &iter);
-                                                                                
 }
 
 
@@ -228,14 +216,14 @@ void
 on_druid_about_clicked (GtkWidget *button, gpointer data)
 {
 	static GtkWidget *about;
-	GdkPixbuf *pixbuf = NULL;
+
 	static const char *authors[] = {
 		"Jacob Berkman  <jacob@bug-buddy.org>",
 		"Fernando Herrera  <fherrera@onirica.com>",
 		NULL
 	};
 
-	static const char *documentors[] = {
+	static const char *documenters[] = {
 		"Telsa Gwynne  <hobbit@aloss.ukuu.org.uk>",
 		"Kevin Conder  <kevin@kevindumpscore.com>",
 		"Eric Baudais  <baudais@kkpsi.org>",
@@ -256,36 +244,24 @@ on_druid_about_clicked (GtkWidget *button, gpointer data)
 		return;
 	}
 		
-	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   "bug-buddy", 48, 0, NULL);
-	
-	about = gnome_about_new (_("Bug Buddy"), VERSION,
-				 "Copyright \xc2\xa9 1999-2002 Jacob Berkman\n"
-				 "Copyright \xc2\xa9 2000-2002 Ximian, Inc.\n"
-				 "Copyright \xc2\xa9 2003 Fernando Herrera",
-				 _("The graphical bug reporting tool for GNOME."),
-				 authors,
-				 documentors,
-				 (strcmp (translators, "translator_credits-PLEASE_ADD_YOURSELF_HERE")
-					 ? translators : NULL), pixbuf);
+	about = gtk_about_dialog_new ();
+
+	gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about), _("Bug Buddy"));
+	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about), VERSION);
+	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about), _("The graphical bug reporting tool for GNOME."));
+	gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about), _("Copyright \xc2\xa9 1999-2002 Jacob Berkman\n"
+								    "Copyright \xc2\xa9 2000-2002 Ximian, Inc.\n"
+								    "Copyright \xc2\xa9 2003-2005 Fernando Herrera"));
+	gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about), authors);
+	gtk_about_dialog_set_documenters (GTK_ABOUT_DIALOG (about), documenters);
+	gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (about), strcmp (translators, 
+							 "translator_credits-PLEASE_ADD_YOURSELF_HERE") ? translators : NULL);
+	gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about), "bug-buddy");
 
-	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
-	
 	g_signal_connect (G_OBJECT (about), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &about);
 
-#if 0
-	{
-		GtkWidget *href;
-		href = gnome_href_new ("http://bug-buddy.org/",
-				       _("The lame Bug Buddy web page"));
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
-				    href, FALSE, FALSE, 0);
-		gtk_widget_show (href);
-	}
-#endif
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (GET_WIDGET ("druid-window")));
 	gtk_widget_show (about);
@@ -369,12 +345,12 @@ email_is_valid (const char *addy)
 	gboolean is_dot_ok = FALSE;
 	gboolean seen_at  = FALSE;
 	const char *tld = NULL;
-	const unsigned char *s;
+	const char *s;
 	const char **t;
 
 	for (s = addy; *s; s++) {
 		/* Must be in acceptable ASCII range. */
-		if (*s <= 32 || *s >= 128)
+		if ((unsigned char)*s <= 32 || (unsigned char)*s >= 128)
 			return FALSE;
 		if (*s == '.') {
 			/* No dots at start, after at, or two in a row. */
@@ -669,6 +645,10 @@ submit_ok (void)
 	if (error) 
 		g_error_free (error);
 
+	if (druid_data.kill != 0) {
+		kill (druid_data.kill, SIGTERM);
+	}
+
 	return retval;
 }
 
@@ -740,9 +720,9 @@ select_version (GSList *list, char *version)
 
 	if (version && g_slist_find_custom (list, version ,(GCompareFunc)strcmp)!=NULL) {
 		druid_data.version = strdup (version);
-		gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("the-version-menu")),
-					     g_slist_position (list, g_slist_find_custom (list, version, 
-											  (GCompareFunc)strcmp)));
+		gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("the-version-menu")),
+					  g_slist_position (list, g_slist_find_custom (list, version,
+										       (GCompareFunc)strcmp)));
 		return;
 	}
 	 
@@ -755,18 +735,18 @@ select_version (GSList *list, char *version)
 
 		if (g_slist_find_custom (list, xversion ,(GCompareFunc)strcmp)!=NULL) {
 			druid_data.version = strdup (xversion);
-			gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("the-version-menu")),
-						     g_slist_position (list, g_slist_find_custom (list, xversion, 
-												  (GCompareFunc)strcmp)));
+			gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("the-version-menu")),
+						  g_slist_position (list, g_slist_find_custom (list, xversion,
+											       (GCompareFunc)strcmp)));
 			g_free (xversion);
 			return;
 		}
 		g_free (xversion);
 	}
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("the-version-menu")),
-				     g_slist_position (list, g_slist_find_custom (list, "unspecified", 
-										  (GCompareFunc)strcmp)));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("the-version-menu")),
+				  g_slist_position (list, g_slist_find_custom (list, "unspecified", 
+									       (GCompareFunc)strcmp)));
 }
 
 static void
@@ -774,9 +754,9 @@ select_severity (GSList *list, char *severity)
 {
 	if (severity && g_slist_find_custom (list, severity, (GCompareFunc)strcmp)!=NULL) {
 		druid_data.severity = strdup (severity);
-		gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("severity-list")),
-					     g_slist_position (list, g_slist_find_custom (list, severity,
-											  (GCompareFunc)strcmp)));
+		gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("severity-list")),
+					  g_slist_position (list, g_slist_find_custom (list, severity,
+										       (GCompareFunc)strcmp)));
 	}
 }
 
@@ -811,7 +791,46 @@ on_druid_cancel_clicked (GtkWidget *w, gpointer data)
 gboolean
 enter_intro (void)
 {
+	BugzillaApplication *application;
+	BugzillaProduct *product;
+	BugzillaBTS *bts;
+	char *markup;
+
 	gtk_widget_set_sensitive (GET_WIDGET ("druid-prev"), FALSE);
+
+	if (!druid_data.appname)
+		return TRUE;
+	
+	application = g_hash_table_lookup (druid_data.program_to_application, druid_data.appname);
+	if (!application || !application->bugzilla || !application->product)
+		return TRUE;
+
+	bts = g_hash_table_lookup (druid_data.bugzillas, application->bugzilla);
+	if (!bts)
+       		return TRUE;
+	
+	product = g_hash_table_lookup (bts->products, application->product);
+	if (!product)
+		return TRUE;
+	
+	druid_data.product = product;
+	bugzilla_product_add_components_to_clist (druid_data.product);
+	buddy_set_text ("email-to-entry", druid_data.product->bts->email);
+	druid_data.product_skipped = TRUE;
+	if (application->component) {
+		druid_data.component = g_hash_table_lookup (product->components, application->component);
+		if (druid_data.component) {
+			druid_data.component_skipped = TRUE;
+		}
+	}
+
+	update_version_menu (product);
+	select_version (product->versions, druid_data.version);
+
+	markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", application->name);
+	gtk_label_set_markup (GTK_LABEL (GET_WIDGET ("app-label")), markup);
+	g_free (markup);
+
 	return TRUE;
 }
 
@@ -829,6 +848,7 @@ done_intro (void)
 	} else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("type-debug-button")))) {
 		druid_data.bug_type = BUG_DEBUG;
 	}
+
 	return TRUE;
 }
 
@@ -875,7 +895,7 @@ done_gdb (void)
 	BugzillaProduct *product;
 	BugzillaBTS *bts;
 
-	if (!druid_data.current_appname)
+	if (!druid_data.appname)
 		return TRUE;
 	
 	application = g_hash_table_lookup (druid_data.program_to_application, druid_data.current_appname);
@@ -1026,7 +1046,57 @@ gboolean
 enter_desc (void)
 {
 	if (druid_data.description_filled == FALSE) {
+		GtkTextBuffer *desc_buffer;
+		GtkTextIter iter;
+
 		mail_write_template ();
+		desc_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GET_WIDGET ("desc-text")));
+
+		if (popt_data.include_file && g_file_test (popt_data.include_file, G_FILE_TEST_EXISTS)) {
+			int fd;
+
+			fd = open (popt_data.include_file, O_RDONLY);
+			if (fd == -1) {
+				d(g_warning ("Could not open file '%s' for reading", filename));
+        		} else {
+				char buffer[1024];
+				char *line;
+				int f_read = 0;
+
+				gtk_text_buffer_get_end_iter (desc_buffer, &iter);
+
+				if (g_utf8_validate (popt_data.include_file, -1, NULL)) {
+					line = g_strdup_printf ("\nIncluded file \"%s\":\n", popt_data.include_file);
+				} else {
+					gsize c_read, c_written;
+					gchar *utf8_text = g_filename_to_utf8 (popt_data.include_file, -1, &c_read, &c_written, NULL);
+					line = g_strdup_printf ("\nIncluded file \"%s\":\n", utf8_text);
+					g_free (utf8_text);
+				}
+				gtk_text_buffer_insert (desc_buffer, &iter, line, strlen (line));
+				g_free (line);
+
+				do {
+					f_read = read (fd, buffer, 1023);
+					if (g_utf8_validate (buffer, f_read, NULL)) {
+						gtk_text_buffer_insert (desc_buffer, &iter, buffer, f_read);
+					} else {
+						gsize c_read, c_written;
+						gchar *utf8_text = g_locale_to_utf8 (buffer, f_read, &c_read, &c_written, NULL);
+						gtk_text_buffer_insert (desc_buffer, &iter, utf8_text, c_written);
+						g_free (utf8_text);
+					}
+				} while (f_read > 0);
+
+				close (fd);
+			}
+		}
+
+		/* set the cursor at the beginning of the second line
+	 	 * in the description page */
+		gtk_text_buffer_get_end_iter (desc_buffer, &iter);
+		gtk_text_buffer_get_iter_at_line (desc_buffer, &iter, 1);
+		gtk_text_buffer_place_cursor (desc_buffer, &iter);
 	}
 	druid_data.description_filled = TRUE;
 	return TRUE;
